textarea {
overflow:auto;
}

table.flaglist{
	font-size: 12px;
}

tr.search{
	background-color: #666666;
	color: #FFFFFF;
	font-size: 14px;
	font-weight: bold;
	font-family: Helvetica, Sans-Serif;
}

#records thead tr th {
	/*font-family: Helvetica, Sans-Serif;*/
	/*font-size: 12px;*/
	background-color: #666666;
	font-weight: bold;
	text-align: center;
}

#records thead tr th a:link {
	color: white;
	text-decoration: none;
	font-weight: bold;
	text-align: center;
}

#records thead tr th a:visited {
	color: white;
	text-decoration: none;
	font-weight: bold;
	text-align: center;
}

#records thead tr th a:hover {
	color: #cccccc;
	text-decoration: underline;
	font-weight: bold;
	text-align: center;
}

#records tbody.scrollable {
	overflow: auto;
}

#records tbody tr td {
	color: #003366;
	font-family: Helvetica, Sans-Serif;
	font-size: 12px;
	text-align: center;
}

#records tbody tr.even  td {
      background-color: #e8e8e8;
}